library engine;

import 'dart:async';
import 'dart:html';

import 'package:rxdart/rxdart.dart' as rx;

export 'package:rxdart/rxdart.dart' show Observable, observable;

void bind(String id, Stream<List<Map<String, String>>> stream) {
  final Element element = findElement(document.body, id);
  final Element parent = element.parent;
  final String str = parent.innerHtml.replaceAll('fe-id="$id"', '');
  final List res = scanElement(element);
  final NodeValidatorBuilder htmlValidator = res.first;
  final Map<String, String> mustacheValues = res.last;

  rx.observable(stream)
      .startWith(<List<Map<String, String>>>[<Map<String, String>>[mustacheValues]])
      .listen((List<Map<String, String>> instances) {
    parent.setInnerHtml(instances.map((Map<String, String> values) {
      String body = str;

      values.forEach((String K, String V) => body = body.replaceAll('{{$K}}', V));

      return body;
    }).join('\r'), validator: htmlValidator);
  });
}

Element findElement(Element parent, String id) {
  Element match;

  parent.childNodes
      .where((Node childNode) => childNode is Element)
      .forEach((Element element) {
    element.attributes.forEach((String K, String V) {
      if (K.compareTo('fe-id') == 0 && V.compareTo(id) == 0) match = element;
    });

    Element childResult = findElement(element, id);

    if (childResult != null) match = childResult;
  });

  return match;
}

List scanElement(Element element, [Map<String, String> result, NodeValidatorBuilder validator]) {
  result ??= <String, String>{};
  validator ??= new NodeValidatorBuilder.common();

  element.attributes.forEach((String K, String V) {
    if (isMustache(V)) {
      result[V.substring(2, V.length - 2)] = '';

      validator.allowElement(element.nodeName, attributes: [K]);
    }
  });

  element.childNodes
      .forEach((Node node) {
    if (isMustache(node.nodeValue)) result[node.nodeValue.substring(2, node.nodeValue.length - 2)] = '';

    if (node is Element) scanElement(node, result);
  });

  return [validator, result];
}

bool isMustache(String V) => V != null && V.codeUnitAt(0) == 123 && V.codeUnitAt(1) == 123 && V.codeUnitAt(V.length - 2) == 125 && V.codeUnitAt(V.length - 1) == 125;